// $Id: tagmap.js,v 1.1.2.1 2010/03/26 15:43:48 kartelusm Exp $

/**
 * @file
 * Tagmap integration in Drupal.
 */

/*global $, Drupal, G_NORMAL_MAP, G_SATELLITE_MAP, G_HYBRID_MAP, G_PHYSICAL_MAP, */
/*global GMap2, GLatLng, GSmallZoomControl, GSmallMapControl, GLargeMapControl,  GMapTypeControl, */
/*global GHierarchicalMapTypeControl, GMenuMapTypeControl, GLatLngBounds, GIcon, */



Drupal.tagmap = new function (){
    var gmarkers = [];

    this.getMarker = function(id) {
        return gmarkers[id];
    };

    this.numberOfMarkers = function() {
        return gmarkers.length;
    };

    this.handleSelected = function (opt) {
        var indexNo = opt[opt.selectedIndex].index;
        GEvent.trigger(gmarkers[indexNo], "click");

    };

    /**
     *  Dropdown list with the tags as an option. On click, go to the tag on the map and open the info
     */
    this.renderOptions = function (mapid) {
        if (!Drupal.settings.tagmap[mapid].clustermarker){
            $('.tagmap-control').parent().append("<div class=\"tagmap-select-div\"></div>")
            $('.tagmap-select-div').append("<select onchange=\"Drupal.tagmap.handleSelected(this);\" class=\"tagmap-select-select-options\"></select>")
            var len = Drupal.tagmap.numberOfMarkers();
            //alert (len);
            for (var i = 0; i < len; i++){
                $('.tagmap-select-select-options').append("<option  value=\""+i+"\">"+Drupal.tagmap.getMarker(i).text_+"</option>")
            }
        }
    };

    /**
     * Encapsulation of marker creation. Needed so that events trigger correctly
     */
    this.createMarker = function (lat, lon, opts, text){
        var marker = new TextMarker(new GLatLng(lat, lon), opts);
        GEvent.addListener(marker, "click", function() {
            marker.openInfoWindowHtml(text);
        });
        return marker;
    };

    this.init = function () {
        var obj = this;
        if (GBrowserIsCompatible()) {
            var mapid = obj.id.split('-');
            mapid.pop();
            mapid.shift();
            mapid = mapid.join('-');
            obj.vars = Drupal.settings.tagmap[mapid];
            obj.vars.initial_state.zoom = parseInt(obj.vars.initial_state.zoom);
            var mapTypes = {
                'map':G_NORMAL_MAP,
                'satellite':G_SATELLITE_MAP,
                'hybrid':G_HYBRID_MAP,
                'physical':G_PHYSICAL_MAP
            };
            obj.vars.mapTypes = [];
            for (layer in obj.vars.baselayers) {
                obj.vars.mapTypes.push(mapTypes[layer]);
            }
            var map = new GMap2(document.getElementById(obj.id), obj.vars);
	  
            //Setting the map center. Check if user provided the default center coordinates,
            // otherwise map center will be calculated on the basis of the markers position on the map.
            var center = new GLatLng(0,0);
            if (obj.vars.initial_state.center) {
                center = obj.vars.initial_state.center.split(',');
                center = new GLatLng(center[0], center[1]);
            }
            map.setCenter(center);
	  	  
            // Setting map control type
            if (obj.vars.controltype === 'Micro') {
                map.addControl(new GSmallZoomControl());
            }
            else if (obj.vars.controltype === 'Small') {
                map.addControl(new GSmallMapControl());
            }
            else if (obj.vars.controltype === 'Large') {
                map.addControl(new GLargeMapControl());
            }
            //Setting map menu type control
            if (obj.vars.mtc === 'standard') {
                map.addControl(new GMapTypeControl());
            }
            else if (obj.vars.mtc === 'hier') {
                map.addControl(new GHierarchicalMapTypeControl());
            }
            else if (obj.vars.mtc === 'menu') {
                map.addControl(new GMenuMapTypeControl());
            }
            // Setting scrollwheel and continuous zoom
            if (!obj.vars.behavior.nomousezoom) {
                map.enableScrollWheelZoom();
                map.enableContinuousZoom();
            }
	  
            map.setMapType(mapTypes[obj.vars.maptype]);
	  
            // Generation of Text markers
            var cluster = [];
            var bounds = new GLatLngBounds;
            for (var i=0; i< obj.vars.markers.length; i++) {
                var opts = {};
                opts.text = obj.vars.markers[i].term;
                opts.weight = obj.vars.markers[i].weight;
                opts.icon = new GIcon(G_DEFAULT_ICON);

                //If a path to the image is in the database, render the thumbnail image...
                if(obj.vars.markers[i].imageField != ''){
                    style = "";
                    if (obj.vars.markers[i].imageThumbWidth != ''){
                        style = "width:"+obj.vars.markers[i].imageThumbWidth+"; ";
                    }
                    if (obj.vars.markers[i].imageThumbHeight != ''){
                        style += "height:"+obj.vars.markers[i].imageThumbHeight+"; ";
                    }
                    imageThumb = '<div id="tagmap-image-'+i+'" style="'+style+'" class="image"><img src="'+obj.vars.markers[i].imageThumbPath+'"></img></div>';
                    opts.infoText = imageThumb+obj.vars.markers[i].infoText;
                }
                //...or not
                else {
                    opts.infoText = obj.vars.markers[i].infoText;
                }
                obj.vars.markers[i].infoText = opts.infoText;
                opts.minFontSize = obj.vars.markers[i].minFontSize;
                opts.fontStyle = obj.vars.markers[i].fontStyle;
                var marker = Drupal.tagmap.createMarker (obj.vars.markers[i].latitude, obj.vars.markers[i].longitude, opts, obj.vars.markers[i].infoText);
                gmarkers.push(marker);
                if (obj.vars.clustermarker) {
                    cluster.push(marker);
                }
                else {
                    map.addOverlay(marker);
                }
                
                bounds.extend(marker.getPoint());
            }
            if (obj.vars.clustermarker) {
                MarkerClusterer(map, cluster);
            }

            // If the default zoom level isn't provided, calculate the zoom level based on
            // the markers position on the map, otherwise use the provided value.
            if (obj.vars.initial_state.zoom == -1){
                map.setZoom(map.getBoundsZoomLevel(bounds));
            }
            else {
                map.setZoom(obj.vars.initial_state.zoom);
            }
	  
            // If the default map center coordinates aren't provided set
            // the map center based on the markers position on the map.
            if (!obj.vars.initial_state.center) {
                map.setCenter(bounds.getCenter());
            }
            Drupal.tagmap.renderOptions(mapid);
        }
    };
};

Drupal.behaviors.tagmap = function(context) {
    $('.tagmap-control:not(.tagmap-processed)', context).addClass('tagmap-processed').each(Drupal.tagmap.init);

}


