<?php
// $Id: views_plugin_style_tagmap.inc,v 1.1.2.5 2010/04/22 11:23:53 kartelusm Exp $

/**
 *  @file
 *
 * Style plugin to render a tagmap.
 *
 * @ingroup views_style_plugins
 */
class views_plugin_style_tagmap extends views_plugin_style {
/**
 * Set default options
 */
  function option_definition() {
    $options = parent::option_definition();
    $options['steps'] = array('default' => '6');
    return $options;
  }

  function query() {
    $this->real_field = ($this->options['tagfield'] ? $this->options['tagfield'] : 'name');
    $this->field_alias = $this->view->query->add_field($this->view->base_table, $this->real_field);

    $this->view->query->add_orderby($this->view->base_table, $this->real_field, 'ASC');
  }


  function render($values) {
    if (empty($this->row_plugin)) {
      vpr('Missing row plugin');
      return;
    }
    // Determine fieldname for latitude and longitude fields.
    $lat_field = $this->view->display_handler->get_handler('field', $this->options['latfield'])->field_alias;
    $lon_field = $this->view->display_handler->get_handler('field', $this->options['lonfield'])->field_alias;

    $tid_field = 'tid';

    // Determine fieldname for term name field.
    $term_name_field = ($this->view->display_handler->get_handler('field', $this->options['tagfield'])->field_alias ? $this->view->display_handler->get_handler('field', $this->options['tagfield'])->field_alias : 'term_data_name');

    // Determine fieldname for term weight field.
    if ($this->options['weightfield']) {
      $weight_field = $this->view->display_handler->get_handler('field', $this->options['weightfield'])->field_alias;
    }

    // Generate markers from view resultset
    $output = '';
    $markers = array();

    //Sets the dimensions of the image to be rendered. Only if ImageCache module is present. Pixels only.
    $image_field = ''; $imagecache_namespace = ''; $width; $height;
    if (module_exists('imagecache') && $this->options['imagecache_icon'] != '--' && $this->options['imagefield'] && $this->options['imagefield'] != '--') {
      $image_field = $this->view->display_handler->get_handler('field', $this->options['imagefield'])->field_alias;
      $imagecache_namespace = $this->view->style_options['imagecache_icon'];
      $preset_actions = imagecache_preset_actions(imagecache_preset_by_name($imagecache_namespace));
      foreach ($preset_actions as $preset) {
        if (isset($preset['data']['width']) && count(explode($preset['data']['width'], '%')) == 1) {
          if (!isset($width) || ((int)$width) < ((int)$preset['data']['width']))
            $width = $preset['data']['width'];
        }
        if (isset($preset['data']['height']) && count(explode($preset['data']['height'], '%')) == 1) {
          if (!isset($height) || ((int)$height) < ((int)$preset['data']['height']))
            $height = $preset['data']['height'];
        }
      }
      if (isset($width)) $width .= 'px';
      if (isset($height)) $height .= 'px';
    }
    if (count($this->view->result) != 0)
      $this->row_plugin->render($this->view->result[0]);
      
    $this->view->row_index = 0;
    
    foreach ($this->view->result as $label => $row) {
      //print_r ($row);
      if (module_exists('imagecache') && $this->options['imagecache_icon'] != '--' && $this->options['imagefield'] && $this->options['imagefield'] !='--') {
        $image_full_path = $row->{$image_field};
        $image_thumb_path = base_path() . imagecache_create_path($imagecache_namespace, $row->{$image_field});
        unset($row->{$image_field});
      }

      $lat = (float)$row->{$lat_field};
      $lon = (float)$row->{$lon_field};
      $tid = $row->{$tid_field};
      $term_name = $row->{$term_name_field};

      if (!$weight_field) {
        $weight = tagmap_term_count_nodes($row->{$tid_field});
      }
      else {
        $weight = $row->{$weight_field};
      }
      //if ($weight && !empty($lat) && !empty($lon)) {
      if ($weight) {
        $markers[] = array(
            'latitude' => $lat,
            'longitude' => $lon,
            'term' => $term_name,
            'weight' => $weight,
            'imageField' => isset($image_field) ? $image_field : '',
            'imageThumbPath' => isset($image_thumb_path) ? $image_thumb_path : '',
            'imageThumbWidth' => isset($width) ? $width : '',
            'imageThumbHeight' => isset($height) ? $height : '',
            'imageFullPath' => isset($image_full_path) ? $image_full_path : '',
            'infoText' => $this->row_plugin->render($row),
            'minFontSize' => $this->options['fontsize'],
            'fontStyle' => $this->options['fontstyle']
        );
      }
      else {
        if (empty($weight)) {
          drupal_set_message(t('The tagmap could not be shown because a weight field is not set.'), 'error');
        }
      }
      $this->view->row_index++;
      
    }
    tagmap_generate_weighted_markers($markers, $this->view->style_plugin->options['steps']);

    // If there are no markers, don't draw empty map.
    if (!empty($markers)) {
      $map['markers'] = $markers;
      if (isset($this->view->style_plugin->options['map_settings']['width'])) {
        $map['width'] = $this->view->style_plugin->options['map_settings']['width'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['height'])) {
        $map['height'] = $this->view->style_plugin->options['map_settings']['height'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['clustermarker'])) {
        $map['clustermarker'] = $this->view->style_plugin->options['map_settings']['clustermarker'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['initial_state']['zoom'])) {
        $map['initial_state']['zoom'] = $this->view->style_plugin->options['map_settings']['initial_state']['zoom'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['initial_state']['center'])) {
        $map['initial_state']['center'] = $this->view->style_plugin->options['map_settings']['initial_state']['center'];
      }
      $output .= theme($this->theme_functions(), $this->view, $this->options, $map, $title);
    }
    else {
      drupal_set_message(t('The tagmap has no markers to display. Choose some content to see the tagmap.'), 'status');
    }
    return $output;
  }

  /**
   * Render the given style.
   */
  function options_form(&$form, &$form_state) {
    $defaults = array_merge(tagmap_defaults(), variable_get('tagmap_settings', array()));
    $form['map_settings'] = array(
        '#type' => 'fieldset',
        '#title' => t('Map settings'),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
    );

    $form['map_settings']['width'] = array(
        '#type' => 'textfield',
        '#title' => t('Width'),
        '#description' => t('Map width, as a CSS length or percentage. Examples: <em>50px</em>, <em>5em</em>, <em>2.5in</em>, <em>95%</em>.'),
        '#default_value' => (isset($this->options['map_settings']['width']) ? $this->options['map_settings']['width'] : $defaults['width']),
        '#maxlength' => 10,
        '#size' => 10,
        '#required' => TRUE,
    );

    $form['map_settings']['height'] = array(
        '#type' => 'textfield',
        '#title' => t('Height'),
        '#description' => t('Map height, as a CSS length or percentage. Examples: <em>50px</em>, <em>5em</em>, <em>2.5in</em>, <em>95%</em>.'),
        '#default_value' => (isset($this->options['map_settings']['height']) ? $this->options['map_settings']['height'] : $defaults['height']),
        '#maxlength' => 10,
        '#size' => 10,
        '#required' => TRUE,
    );

    $form['map_settings']['initial_state'] = array(
        '#type' => 'fieldset',
        '#title' => t('Set the initial state of the map'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
        '#description' => t('These settings are optional. If you do not enter the values, they will be calculated on the basis of the markers position on the map.'),
    );

    $form['map_settings']['initial_state']['zoom'] = array(
        '#type' => 'select',
        '#title' => t('Default zoom'),
        '#default_value' => (isset($this->options['map_settings']['initial_state']['zoom']) ? $this->options['map_settings']['initial_state']['zoom'] : $defaults['initial_state']['zoom']),
        '#options' => array(-1 => t('None')) + drupal_map_assoc(range(0, 17)),
        '#description' => t('The default zoom level of the map.'),
    );

    $form['map_settings']['initial_state']['center'] = array(
        '#type' => 'textfield',
        '#title' => t('Default center'),
        '#default_value' => (isset($this->options['map_settings']['initial_state']['center']) ? $this->options['map_settings']['initial_state']['center'] : $defaults['initial_state']['center']),
        '#size' => 50,
        '#maxlength' => 255,
        '#description' => t('The default center coordinates of the map, expressed as a decimal latitude and longitude, separated by a comma.'),
        '#element_validate' => array('tagmap_center_validate'),
    );

    if (file_exists(drupal_get_path('module', 'tagmap') .'/js/markercluster_packed.js')) {
      $form['map_settings']['clustermarker'] = array(
          '#type' => 'checkbox',
          '#title' => t('Use cluster for marker management'),
          '#default_value' => (isset($this->options['map_settings']['clustermarker']) ? $this->options['map_settings']['clustermarker'] : $defaults['clustermarker']),
      );
    }

    $form['steps'] = array(
        '#type' => 'textfield',
        '#title' => t('Tag levels'),
        '#description' => t('Number of different tag levels that will be generated. Must be an integer value.'),
        '#default_value' => (isset($this->options['steps']) ? $this->options['steps'] : 6),
        '#maxlength' => 10,
        '#size' => 10,
        '#required' => TRUE,
    );

    $form['fontsize'] = array(
        '#type' => 'textfield',
        '#title' => t('Font size in pixels'),
        '#description' => t('Minimum font size for the first level terms.'),
        '#default_value' => (isset($this->options['fontsize']) ? $this->options['fontsize'] : 10),
        '#maxlength' => 10,
        '#size' => 10,
        '#required' => TRUE,
    );
    
    $font_style = array('normal' => 'Normal', 'italic' => 'Italic', 'oblique' => 'Oblique');
    
    $form['fontstyle'] = array(
        '#type' => 'select',
        '#title' => t('Font style'),
        '#default_value' => (isset($this->options['fontstyle']) ? $this->options['fontstyle'] : $font_style['normal']),
        '#options' => $font_style,
        '#required' => TRUE,
    );    

    $supported_string_handlers = array('views_handler_field' , 'views_handler_field_taxonomy');
    $supported_numeric_handlers = array('views_handler_field_numeric', 'geotaxonomy_handler_field_term_node_count');
    $options = array('numerical' => array(), 'string' => array());
    $handlers = $this->display->handler->get_handlers('field');
    foreach ($handlers as $field => $handler) {
      $handler_class = get_class($handler);
      if (in_array($handler_class, $supported_numeric_handlers)) {
        $options['numerical'][$field] = $handler->ui_name();
      }
      elseif (in_array($handler_class, $supported_string_handlers)) {
        $options['string'][$field] = $handler->ui_name();
      }
    }
    
    //This step is necessary because TW exports to views everything as String.
    //I will upload a patch for TW if someone didn't do it already
    if ($this->view->base_table != 'term_data')
        $options['numerical'] = $options['string'];

    $form['latfield'] = array(
        '#title' => t('Latitude field'),
        '#description' => t('Format must be degrees decimal.'),
        '#type' => 'select',
        '#options' => $options['numerical'],
        '#default_value' => $this->options['latfield'],
        '#required' => TRUE,
    );

    $form['lonfield'] = array(
        '#title' => t('Longitude field'),
        '#description' => t('Format must be degrees decimal.'),
        '#type' => 'select',
        '#options' => $options['numerical'],
        '#default_value' => $this->options['lonfield'],
        '#required' => TRUE,
    );

    $form['weightfield'] = array(
        '#title' => t('Tag weight field'),
        '#description' => t('If you don\'t choose a field, term weight will be calculated counting number of nodes assigned with the term.'),
        '#type' => 'select',
        '#options' => ($this->view->base_table != 'term_data' ? $options['numerical'] : array_merge(array('' => 'None'), $options['numerical'])),
        '#default_value' => $this->options['weightfield'],
    );

    if ($this->view->base_table != 'term_data') {
      $form['tagfield'] = array(
          '#title' => t('Tag field'),
          '#description' => t('String that will be displayed over map.'),
          '#type' => 'select',
          '#options' => array_merge(array('' => ''), $options['string']),
          '#default_value' => $this->options['tagfield'],
          '#required' => TRUE,
      );

      if (module_exists('imagecache')) {
        $imagecache_presets = array('--' => t('--'));

        foreach (imagecache_presets() as $preset) {
          $imagecache_presets[$preset['presetname']] = $preset['presetname'];
        }

        $form['imagecache_icon'] = array(
            '#type' => 'select',
            '#title' => t('Imagecache preset for Image'),
            '#options' => $imagecache_presets,
            '#description' => t('Imagecache preeset. image dimensions should be in px. Not necessary, but recomended.'),
            '#default_value' => $this->options['imagecache_icon'],
        );

        $form['imagefield'] = array(
            '#title' => t('Image field'),
            '#type' => 'select',
            '#options' => array_merge(array('--' => '--'), $options['string']),
            '#default_value' => $this->options['imagefield'],
            '#description' => t('This field should be excluded from display so the path to the image doesn\'t render.'),
        );
      }
    }
  }

  function options_validate(&$form, &$form_state) {
    parent::options_validate($form, $form_state);

    if (!is_numeric($form_state['values']['style_options']['steps'])) {
      form_set_error('style_options][steps', t('Field @field must be integer.', array('@field' => $form['steps']['#title'])));
    }
    elseif ($form_state['values']['style_options']['steps'] <= 0) {
      form_set_error('style_options][steps', t('Field @field must be positive value.', array('@field' => $form['steps']['#title'])));
    }

    if (!is_numeric($form_state['values']['style_options']['fontsize'])) {
      form_set_error('style_options][fontsize', t('Field @field must be numeric.', array('@field' => $form['fontsize']['#title'])));
    }
    elseif ($form_state['values']['style_options']['fontsize'] <= 0) {
      form_set_error('style_options][fontsize', t('Field @field must be positive value.', array('@field' => $form['fontsize']['#title'])));
    }

    if (!tagmap_css_dimensions_validate($form_state['values']['style_options']['map_settings']['width'])) {
      form_set_error('style_options][map_settings][width', t('The specified value is not a valid CSS dimension.'));
    }

    if (!tagmap_css_dimensions_validate($form_state['values']['style_options']['map_settings']['height'], TRUE)) {
      form_set_error('style_options][map_settings][height', t('The specified value is not a valid CSS dimension or is a percentage.'));
    }
  }
}